==> rx-0.1.1.tar.gz: lib/src/operators/buffer.dart <==
library rx.operators.buffer;

import '../core/observable.dart';
import '../core/observer.dart';
import '../core/subscriber.dart';
import '../disposables/disposable.dart';
import '../observers/inner.dart';
import '../schedulers/scheduler.dart';
import '../schedulers/settings.dart';

extension BufferOperator<T> on Observable<T> {
  /// Gathers the items emitted by this [Observable] and bundles these items
  /// into a list until either
  ///
  /// - another [Observable] [trigger]s,
  /// - the buffer reaches [maxLength], or
  /// - the buffer reaches [maxAge].
  ///
  Observable<List<T>> buffer(
          {Scheduler? scheduler,
          Observable? trigger,
          int? maxLength,
          Duration? maxAge}) =>
      BufferObservable<T>(
          this, scheduler ?? defaultScheduler, trigger, maxLength, maxAge);
}

class BufferObservable<T> extends Observable<List<T>> {
  final Observable<T> delegate;
  final Scheduler scheduler;
  final Observable? trigger;
  final int? maxLength;
  final Duration? maxAge;

  BufferObservable(
      this.delegate, this.scheduler, this.trigger, this.maxLength, this.maxAge);

  @override
  Disposable subscribe(Observer<List<T>> observer) {
    final subscriber =
        BufferSubscriber<T>(observer, scheduler, trigger, maxLength, maxAge);
    subscriber.add(delegate.subscribe(subscriber));
    return subscriber;
  }
}

class BufferSubscriber<T> extends Subscriber<T>
    implements InnerEvents<T, void> {
  final Scheduler scheduler;
  final int? maxLength;
  final Duration? maxAge;

  List<T> buffer = const [];
  DateTime? bufferBirth;

  BufferSubscriber(Observer<List<T>> observer, this.scheduler,
      Observable? trigger, this.maxLength, this.maxAge)
      : super(observer) {
    reset();
    if (trigger != null) {
      add(InnerObserver(this, trigger, null));
    }
  }

  @override
  void onNext(T value) {
    buffer.add(value);
    bufferBirth ??= scheduler.now;
    if (shouldFlush) {
      flush();
    }
  }

  @override
  void onComplete() {
    flush();
    doComplete();
  }

  @override
  void notifyNext(Disposable disposable, void state, T value) => flush();

  @override
  void notifyError(Disposable disposable, void state, Object error,
          StackTrace stackTrace) =>
      doError(error, stackTrace);

  @override
  void notifyComplete(Disposable disposable, void state) {}

  bool get shouldFlush =>
      (maxLength != null && maxLength! <= buffer.length) ||
      (maxAge != null &&
          bufferBirth != null &&
          bufferBirth!.add(maxAge!).isBefore(scheduler.now));

  void flush() {
    if (buffer.isNotEmpty) {
      doNext(buffer);
      reset();
    }
  }

  void reset() {
    buffer = [];
    bufferBirth = null;
  }
}

==> rx-0.1.2.tar.gz: lib/src/operators/buffer.dart <==
import '../core/observable.dart';
import '../core/observer.dart';
import '../core/subscriber.dart';
import '../disposables/disposable.dart';
import '../observers/inner.dart';
import '../schedulers/scheduler.dart';
import '../schedulers/settings.dart';

extension BufferOperator<T> on Observable<T> {
  /// Gathers the items emitted by this [Observable] and bundles these items
  /// into a list until either
  ///
  /// - another [Observable] [trigger]s,
  /// - the buffer reaches [maxLength], or
  /// - the buffer reaches [maxAge].
  ///
  Observable<List<T>> buffer<R>(
          {Scheduler? scheduler,
          Observable<R>? trigger,
          int? maxLength,
          Duration? maxAge}) =>
      BufferObservable<T, R>(
          this, scheduler ?? defaultScheduler, trigger, maxLength, maxAge);
}

class BufferObservable<T, R> with Observable<List<T>> {
  final Observable<T> delegate;
  final Scheduler scheduler;
  final Observable<R>? trigger;
  final int? maxLength;
  final Duration? maxAge;

  BufferObservable(
      this.delegate, this.scheduler, this.trigger, this.maxLength, this.maxAge);

  @override
  Disposable subscribe(Observer<List<T>> observer) {
    final subscriber =
        BufferSubscriber<T, R>(observer, scheduler, trigger, maxLength, maxAge);
    subscriber.add(delegate.subscribe(subscriber));
    return subscriber;
  }
}

class BufferSubscriber<T, R> extends Subscriber<T>
    implements InnerEvents<R, void> {
  final Scheduler scheduler;
  final int? maxLength;
  final Duration? maxAge;

  List<T> buffer = const [];
  DateTime? bufferBirth;

  BufferSubscriber(Observer<List<T>> observer, this.scheduler,
      Observable<R>? trigger, this.maxLength, this.maxAge)
      : super(observer) {
    reset();
    if (trigger != null) {
      add(InnerObserver<R, void>(this, trigger, null));
    }
  }

  @override
  void onNext(T value) {
    buffer.add(value);
    bufferBirth ??= scheduler.now;
    if (shouldFlush) {
      flush();
    }
  }

  @override
  void onComplete() {
    flush();
    doComplete();
  }

  @override
  void notifyNext(Disposable disposable, void state, R value) => flush();

  @override
  void notifyError(Disposable disposable, void state, Object error,
          StackTrace stackTrace) =>
      doError(error, stackTrace);

  @override
  void notifyComplete(Disposable disposable, void state) {}

  bool get shouldFlush =>
      (maxLength != null && maxLength! <= buffer.length) ||
      (maxAge != null &&
          bufferBirth != null &&
          bufferBirth!.add(maxAge!).isBefore(scheduler.now));

  void flush() {
    if (buffer.isNotEmpty) {
      doNext(buffer);
      reset();
    }
  }

  void reset() {
    buffer = [];
    bufferBirth = null;
  }
}
